'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _objectSpread = _interopDefault(require('@babel/runtime/helpers/objectSpread'));
var _objectWithoutProperties = _interopDefault(require('@babel/runtime/helpers/objectWithoutProperties'));
var _classCallCheck = _interopDefault(require('@babel/runtime/helpers/classCallCheck'));
var _createClass = _interopDefault(require('@babel/runtime/helpers/createClass'));
var _possibleConstructorReturn = _interopDefault(require('@babel/runtime/helpers/possibleConstructorReturn'));
var _getPrototypeOf = _interopDefault(require('@babel/runtime/helpers/getPrototypeOf'));
var _assertThisInitialized = _interopDefault(require('@babel/runtime/helpers/assertThisInitialized'));
var _inherits = _interopDefault(require('@babel/runtime/helpers/inherits'));
var react = require('react');
var reactDom = require('react-dom');
var assignProps = require('@semcore/utils/lib/assignProps');
var setRef = _interopDefault(require('@semcore/utils/lib/setRef'));
var fire = _interopDefault(require('@semcore/utils/lib/fire'));

var KEY = {
  DOWN: 40,
  UP: 38,
  ENTER: 13,
  SPACE: 32
};

var HighlightFromGroup =
/*#__PURE__*/
function (_PureComponent) {
  _inherits(HighlightFromGroup, _PureComponent);

  function HighlightFromGroup() {
    var _this;

    _classCallCheck(this, HighlightFromGroup);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(HighlightFromGroup).apply(this, arguments));
    _this.highlightedNode = null;
    _this.listNode = null;
    _this.items = [];

    _this.selectItemByIndex = function (index) {
      var item = _this.items.find(function (item) {
        return item.index === index;
      });

      if (item) {
        fire(_assertThisInitialized(_this), 'onSelect', item);
      }
    };

    _this.handleKeyDown = function (e) {
      var itemCount = _this.getItemCount();

      var highlightedIndex = _this.props.highlightedIndex;

      if (itemCount) {
        if (e.keyCode === KEY.DOWN) {
          e.preventDefault();
          var amount = e.shiftKey ? 5 : 1;

          _this.moveHighlightedIndex(amount);
        }

        if (e.keyCode === KEY.UP) {
          e.preventDefault();

          var _amount = e.shiftKey ? -5 : -1;

          _this.moveHighlightedIndex(_amount);
        }

        if (e.keyCode === KEY.ENTER || e.keyCode === KEY.SPACE) {
          if (highlightedIndex !== undefined) {
            e.preventDefault();

            _this.selectItemByIndex(highlightedIndex);
          }
        }
      }
    };

    _this.clearItems = function () {
      _this.items = [];
    };

    _this.getItemCount = function () {
      return _this.props.itemCount !== undefined ? _this.props.itemCount : _this.items.length;
    };

    _this.getTriggerProps = function (props) {
      if (!props) props = {};

      var _props = props,
          onKeyDown = _props.onKeyDown,
          onBlur = _props.onBlur,
          other = _objectWithoutProperties(_props, ["onKeyDown", "onBlur"]);

      return _objectSpread({}, other, {
        onKeyDown: assignProps.callAllEventHandlers(onKeyDown, _this.handleKeyDown),
        onBlur: assignProps.callAllEventHandlers(onBlur, function () {
          return _this.props.onHighlightedIndexChange(undefined);
        })
      });
    };

    _this.getItemProps = function (props) {
      if (!props) props = {};
      var highlightedIndex = _this.props.highlightedIndex;
      var _props2 = props,
          index = _props2.index;

      if (index === undefined) {
        index = _this.getItemCount();

        _this.items.push(_objectSpread({}, props, {
          index: index
        }));
      } else {
        _this.items[index] = props;
      }

      var newProps = _objectSpread({
        index: index
      }, props);

      if (index === highlightedIndex) {
        newProps.ref = function (ref) {
          setRef(props.ref, ref);
          _this.highlightedNode = reactDom.findDOMNode(ref);
        };
      }

      return newProps;
    };

    _this.getListProps = function (props) {
      if (!props) props = {};
      return _objectSpread({}, props, {
        ref: function ref(_ref) {
          setRef(props['ref'], _ref);
          _this.listNode = reactDom.findDOMNode(_ref);
        }
      });
    };

    return _this;
  }

  _createClass(HighlightFromGroup, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      if (this.props.highlightedIndex !== undefined) {
        this.scrollHighlightedItemIntoView();
      }
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps) {
      if (this.props.highlightedIndex !== prevProps.highlightedIndex) {
        this.scrollHighlightedItemIntoView();
      }
    }
  }, {
    key: "scrollHighlightedItemIntoView",
    value: function scrollHighlightedItemIntoView() {
      if (this.highlightedNode && this.listNode) {
        var scrollTop = this.listNode.scrollTop;
        var scrollBottom = scrollTop + this.listNode.offsetHeight;
        var optionTop = this.highlightedNode.offsetTop;
        var optionBottom = optionTop + this.highlightedNode.offsetHeight;

        if (scrollTop > optionTop || scrollBottom < optionBottom) {
          this.listNode.scrollTop = optionTop;
        }
      }
    }
  }, {
    key: "moveHighlightedIndex",
    value: function moveHighlightedIndex(amount) {
      var highlightedIndex = this.props.highlightedIndex;
      var itemsLastIndex = this.getItemCount() - 1;

      if (itemsLastIndex < 0) {
        return;
      }

      if (highlightedIndex === undefined) {
        highlightedIndex = amount < 0 ? 0 : itemsLastIndex;
      }

      var newIndex = highlightedIndex + amount;

      if (newIndex < 0) {
        newIndex = itemsLastIndex;
      } else if (newIndex > itemsLastIndex) {
        newIndex = 0;
      }

      if (this.items[newIndex] && this.items[newIndex].disabled) {
        this.moveHighlightedIndex(amount < 0 ? amount - 1 : amount + 1);
        return;
      }

      fire(this, 'onHighlightedIndexChange', newIndex);
    }
  }, {
    key: "render",
    value: function render() {
      var _this$props = this.props,
          children = _this$props.children,
          highlightedIndex = _this$props.highlightedIndex;
      this.clearItems();
      return children({
        getTriggerProps: this.getTriggerProps,
        getItemProps: this.getItemProps,
        getListProps: this.getListProps,
        selectItemByIndex: this.selectItemByIndex,
        highlightedIndex: highlightedIndex,
        getItemCount: this.getItemCount,
        clearItems: this.clearItems
      });
    }
  }]);

  return HighlightFromGroup;
}(react.PureComponent);

HighlightFromGroup.displayName = 'HighlightFromGroup';

exports.default = HighlightFromGroup;
//# sourceMappingURL=index.js.map
